# Thales Custom Dev for XiVO in SMM

Custom dialplan & scripts for XiVO for Thales SMM product.

## Dev

For the dev we use a debian container with all necessary dependencies/python version etc.
To launch it we use this bash function that you can put in your `.bashrc`:
```
function debuilder() {
  local lts=${1:-jabbah}

  docker pull xivoxc/debian-builder-${lts}:latest
  docker run --rm -it \
    --network host \
    -v /etc/localtime:/etc/localtime:ro \
    -v /etc/timezone:/etc/timezone:ro \
    --user "`id -u`:`id -g`" \
    -v /etc/group:/etc/group:ro \
    -v /etc/passwd:/etc/passwd:ro \
    -v /etc/shadow:/etc/shadow:ro \
    -v "`pwd`/..":"`pwd`/.." \
    -w "`pwd`" \
    -e DOCKERIZED=true \
    -e DEBFULLNAME="`git config user.name`" \
    -e DEBEMAIL="`git config user.email`" \
    xivoxc/debian-builder-${lts}:latest
}
```

Then you can launch it :
```
debuilder jabbah
```
### Unit test

To launch unit tests

Launch debian builder:
```
debuilder jabbah
```

Then inside the container 

1. create virtual env and install requirements:
   ```
   python3 -m venv /tmp/venv3 && source /tmp/venv3/bin/activate
   pip install -r requirements.txt -r tests-requirements.txt
   ```
1. And launch tests:
   ```
   nosetests tests
   ```
## Build 

Launch the container at the root of the project:
```
debuilder jabbah
```

Inside the container run command to build deb package: `debuild -us -uc`

## Install

- **install conf on xivo**:
  - send the .deb file on xivo
  - and run the install command: `dpkg -i thales-xivo-custom_2022.06.00_all.deb`


See the [wiki documentation](https://gitlab.com/avencall/thales-xivo-custom/-/wikis/home) for more details.

## Bouchons

For the dev we need several "bouchons".
Some are supplied by Thales:
- ACS
- DNS

But we needed to create some others.
Currently :
- MDT: see bouchons/README.md

## Notes

### Ideas

#### MDT
Another idea could be to create the following config for mdt. So that incoming call would be handled correctly.
```
[mdt_local]
type = endpoint
context = default
allow = alaw


[mdt_local]
type = identify
endpoint = mdt_local
match = 127.0.0.1:45060
```

## Asterisk and Custom codecs

Specific notes for the asterisk specific codecs

### Install specific asterisk version

1. Download the following packages:
   - [asterisk](http://mirror.xivo.solutions/archive/pool/main/a/asterisk/asterisk_18.3.0-1~xivo2+deb10+20211103.163109.4178715_amd64.deb)
   - [asterisk-dbg](http://mirror.xivo.solutions/archive/pool/main/a/asterisk/asterisk-dbg_18.3.0-1~xivo2+deb10+20211103.163109.4178715_amd64.deb)
   - [asterisk-sounds](http://mirror.xivo.solutions/archive/pool/main/a/asterisk/asterisk-sounds-main_18.3.0-1~xivo2+deb10+20211103.163109.4178715_all.deb)
2. Copy the 3 .deb files to the XiVO in `/var/tmp/` directory
3. Install the packages:
   1. Login in xivo
   2. Run:
      ```bash
      cd /var/tmp/
      dpkg -i asterisk_18.3.0-1~xivo2+deb10+20211103.163109.4178715_amd64.deb asterisk-dbg_18.3.0-1~xivo2+deb10+20211103.163109.4178715_amd64.deb asterisk-sounds-main_18.3.0-1~xivo2+deb10+20211103.163109.4178715_all.deb
      ```

### Configure XiVO

**1. Create users**

1. Create 2 users Theo1 and Theo2 with a line in context default

**2. Configure the users' line**

**For each** of the created users, edit the line and:
1. go to *Advanced* tab
2. add a line with 
   * *Option:* `disallow`
   * *Value:* `all`
3. add another line with 
   * *Option:* `allow`
   * *Value:* `mvhd,mvbd`
4. Save

**3. Configure XiVO**

1. Go to *SIP Protocol* and in tab *Signaling*
   1. Check *Customize codecs*
   2. Select only `alaw`
   3. Save
2. From XiVO terminal connect to asterisk db and add mvbd and mvhd to allowed codec (**N.B.** this change will be overwritten by ny change in the XiVO webi *SIP Protocol* menu):
   ```bash
   psql -U asterisk
   update staticsip set var_val = CONCAT(var_val, ',mvbd,mvhd') where var_name = 'allow';
   ```
3. From XiVO terminal reload sip conf:
   ```
   asterisk -rx 'sip reload'
   ```

### Test

#### With the specific phones

Given you have the specific phones:

1. Configure the specific phones to register as Theo1 and Theo2
2. Make calls between Theo1 and Theo2
   * the call should establish
   * you should be able to ear each others

#### Without phones with sipp scenario

Given you don't have the specific phones you can test with sipp scenario:

**1. Configure XiVO**

1. Go to *Contexts* menu
2. Create new context
   * *Name*: `thales-codec`
   * *Display name*: `thales-codec`
   * *Type*: `Other`
3. Go to SIP Protocol
   1. tab *General*, activate *Accept unauthenticated calls*
   2. tab *Default*,
      1. for *Context* option, select `thales-codec`
      2. for *Redirect media streams* option, select **No**
   3. Save
4. Go to *Configuration files*
   1. add new thales-codec configuration file with following content
      ```
      [thales-codec]
      exten = 555,1,Set(TH_PEER=5htuc021)
      same  =     n,Dial(SIP/${TH_PEER})
      ```
    1. **MODIFY** `TH_PEER` value with the Theo1 line name
5. Edit Theo1 line name and go to *Advanced* menu
   1. Add a line with
      * *Option:* `host`
      * *Value:* `LAPTOP_IP` (the IP of your laptop from which you will run the sipp scenario)
   4. add another line with 
      * *Option:* `port`
      * *Value:* `5066`
4. From XiVO bash CLI connect to asterisk db and add mvbd and mvhd to allowed codec (**you need to redo it because it was probably overwritten by other changes in the *SIP Protocol* tab**):
   ```
   psql -U asterisk
   update staticsip set var_val = CONCAT(var_val, ',mvbd,mvhd') where var_name = 'allow';
   ```
5. From XiVO CLI reload sip conf:
   ```
   asterisk -rx 'sip reload'

**2. Run sipp scenario**

1. Install sipp on your laptop (at least v3.6.1 https://github.com/SIPp/sipp/releases/tag/v3.6.1 )
2. From your laptop go to this repo asterisk dir: `cd asterisk-codec/sipp-scenario`
3. Then run the mvbd scenario:
   1. in one terminal the UAS (replcae `LAPTOP_IP` by your laptop IP):
      ```
      path/to/sipp -sf uas-mvbd.xml -m 1 -i LAPTOP_IP -p 5066
      ```
   2. then run in another terminal the UAC (replcae `XIVO_IP` by XiVO IP)::
      ```
      path/to/sipp -sf uac-mvbd.xml -s 555 -m 1 XIVO_IP
      ```
3. Then run the mvhd scenario:
   1. in one terminal the UAS (replcae `LAPTOP_IP` by your laptop IP):
      ```
      path/to/sipp -sf uas-mvhd.xml -m 1 -i LAPTOP_IP -p 5066
      ```
   2. then run in another terminal the UAC (replcae `XIVO_IP` by XiVO IP)::
      ```
      path/to/sipp -sf uac-mvhd.xml -s 555 -m 1 XIVO_IP
      ```
4. It works if
   * you see the call in asterisk CLI wihtout any errors 
   * and that the call is established with some rtp flowing

**3. run re-invite scenario**

1. On XiVO in *SIP Protocol* in tab *Default* for option *Redirect media streams* select **Yes**
2. From XiVO bash CLI connect to asterisk db and add mvbd and mvhd to allowed codec (**you need to redo it because it was probably overwritten by other changes in the *SIP Protocol* tab**):
   ```
   psql -U asterisk
   update staticsip set var_val = CONCAT(var_val, ',mvbd,mvhd') where var_name = 'allow';
   ```
3. From XiVO CLI reload sip conf:
   ```
   asterisk -rx 'sip reload'
4. Then run the re-invite mvbd scenario:
   1. in one terminal the UAS (replcae `LAPTOP_IP` by your laptop IP):
      ```
      path/to/sipp -sf uas-reinvite-mvbd.xml -m 1 -i LAPTOP_IP -p 5066
      ```
   2. then run in another terminal the UAC (replcae `XIVO_IP` by XiVO IP)::
      ```
      path/to/sipp -sf uac-reinvite-mvbd.xml -s 555 -m 1 XIVO_IP
      ```
4. It works if
   * you see the call in asterisk CLI wihtout any errors 
   * and that the call is established and *Remotely bridged*
