#!/usr/bin/python3

import sys
import psycopg2
from xivo.agi import AGI

DB_URI = 'postgresql://userasterisk:password@localhost:5432/agentams'


def with_agi_params(fonction):
    connexion = psycopg2.connect(DB_URI)
    cursor = connexion.cursor()
    fonction(AGI(), cursor, sys.argv[1:])
    connexion.commit()
    cursor.close()
    connexion.close()


def find_peer_by_number(agi, cursor, number, function):
    agi.verbose(f'looking for peer associated to {number}')
    cursor.execute("SELECT peer_sip FROM affiliation WHERE subscriber_number = %s", (number,))
    return function(agi, cursor)


def find_number_by_peer(agi, cursor, peer, function):
    agi.verbose(f'looking for number associated to {peer}')
    cursor.execute("SELECT subscriber_number FROM affiliation WHERE peer_sip = %s", (peer,))
    return function(agi, cursor)


def export_item_to_agi(agi, cursor):
    result = cursor.fetchall()
    if len(result) == 0:
        agi.verbose('item not found')
        agi.set_variable('TH_AFF_OP_SUCCESSFUL', 'NOK')
        return False
    elif len(result) > 1:
        agi.verbose('unique constraint broken')
        agi.set_variable('TH_AFF_OP_SUCCESSFUL', 'NOK')
        return False
    agi.set_variable('TH_AFF_OP_SUCCESSFUL', 'OK')
    agi.set_variable('TH_AFF_RETURNED_VALUE', result.pop()[0])
    return True


def return_item(agi, cursor):
    item = cursor.fetchall()
    agi.verbose(f'item : {item}')
    if len(item) != 0:
        return item.pop()[0]
    return None


def insert(agi, cursor, peer, number):
    agi.verbose(f'subscription for {peer}, {number}')

    #removing duplicatas that would break unique contstraint on insertion
    breaking_peer = find_peer_by_number(agi, cursor, number, return_item)
    if breaking_peer is not None:
        delete(agi, cursor, breaking_peer)
    breaking_number = find_number_by_peer(agi, cursor, peer, return_item)
    if breaking_number is not None:
        delete(agi, cursor, peer)

    cursor.execute("INSERT INTO affiliation VALUES(%s,%s)", (peer, number))


def delete(agi, cursor, peer):
    agi.verbose(f'unsubscription for {peer}')
    cursor.execute("DELETE FROM affiliation WHERE peer_sip = %s", (peer,))


def switch_operation(agi, cursor, args):
    agi.verbose(f"switching operation with args {args}")
    try:
        operation_type = args[0]
    except IndexError:
        agi.verbose('no operation type provided')
        sys.exit(1)
    if operation_type == 'subscription':
        agi.verbose('subscription')
        try:
            peer = args[1]
            number = args[2]
        except IndexError:
            agi.verbose('peer or number not provided for insertion')
            sys.exit(1)
        insert(agi, cursor, peer, number)
    elif operation_type == 'unsubscription':
        try:
            peer = args[1]
        except IndexError:
            agi.verbose('peer not provided for deletion')
        delete(agi, cursor, peer)
    else:
        try:
            peer_or_number = args[1]
        except IndexError:
            agi.verbose('peer nor number not provided for search')
            sys.exit(1)
        if operation_type == 'searchnum':
            if find_number_by_peer(agi, cursor, peer_or_number, export_item_to_agi):
                sys.exit(0)
            sys.exit(1)
        elif operation_type == 'searchpeer':
            if find_peer_by_number(agi, cursor, peer_or_number, export_item_to_agi):
                sys.exit(0)
            sys.exit(1)
        else:
            agi.verbose('operation type not understood')
            sys.exit(1)


if __name__ == '__main__':
    with_agi_params(switch_operation)
