import unittest
from hamcrest import assert_that
from hamcrest import equal_to
import importlib
dns_lookup = importlib.import_module("agi.var.lib.asterisk.agi-bin.dns-lookup")

class TestDnsLookup(unittest.TestCase):

    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        pass

    def test_extract_number_and_domain(self):
        input = "id 37597 opcode QUERY rcode NOERROR flags QR AA RD RA ;\
            QUESTION *.6.4.8.dynamique.local. IN NAPTR ;\
            ANSWER *.6.4.8.dynamique.local. 60 IN NAPTR 100 10 \"u\" \"E2U+sip\" \"!^.*$!sip:846*@10.181.7.21!\" . ;\
            AUTHORITY ;\
            ADDITIONAL"
        expected = ["846*","10.181.7.21"]
        self.assertEqual(dns_lookup.extract_number_and_domain(input),expected)

        input_2 = "id 23930 opcode QUERY rcode NOERROR flags QR AA RD RA ;\
            QUESTION 2.0.0.0.1.7.7.2.dynamique.local. IN NAPTR ;\
            ANSWER 2.0.0.0.1.7.7.2.dynamique.local. 60 IN NAPTR 100 10 \"U\" \"E2U+sip\" \"!^.*$!sip:27710004@10.181.7.21!\" . ;\
            AUTHORITY ;\
            ADDITIONAL"
        expected_2 = ["27710004","10.181.7.21"]
        self.assertEqual(dns_lookup.extract_number_and_domain(input_2),expected_2)

    def test_extract_prefix_and_ip(self):
        input = "Prefix:A1,IP:10.168.0.2,TrunkRate:30"
        expected = ["A1","10.168.0.2"]
        self.assertEqual(dns_lookup.extract_prefix_and_ip(input),expected)

        input2 = "Prefix:A3,IP:10.177.0.2,TrunkRate:30"
        expected2 = ["A3","10.177.0.2"]
        self.assertEqual(dns_lookup.extract_prefix_and_ip(input2),expected2)

    def test_reverse_and_split_number(self):
        input = "846*"
        output = "*.6.4.8"
        self.assertEqual(dns_lookup.reverse_and_split_number(input),output)