#!/usr/bin/python3
# -*- coding: UTF-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object
__version__   = '$Revision$'
__date__      = '$Date$'
__copyright__ = 'Copyright (C) 2021 Avencall'
__author__    = 'AV'


import sys
import requests
from requests.auth import HTTPBasicAuth
import logging
import traceback
import json
import configparser
from xivo.agi import AGI

agi = AGI()

prec_config = configparser.ConfigParser()
prec_config.readfp(open('/etc/thales/thales-routing.conf'))
ARI_SERVER_IP = prec_config.get('ARI', 'SERVER_IP')
ARI_SERVER_PORT = prec_config.get('ARI', 'SERVER_PORT')
SERVER_URL = 'http://'+ARI_SERVER_IP + ':' + ARI_SERVER_PORT
ARI_USER =  prec_config.get('ARI', 'ARI_USER')
ARI_PASS =  prec_config.get('ARI', 'ARI_PASS')

channelsURL = SERVER_URL+'/ari/channels'
precedencesPrio = {'Routine': 1, 'Priority': 2, 'Immediate': 3, 'Flash': 4, 'FlashOverride': 5}

# User configuration
DEBUG_MODE = False
LOGFILE = '/var/log/thales-get-channel-precedence.log'
# timeout of the tcp connection to the webserver (seconds)
CONNECTION_TIMEOUT = 2

# Factory configuration
URL_HEADERS = { 'User-Agent' : 'XiVO Webservice AGI' }

logger = logging.getLogger()


class Syslogger(object):

    def write(self, data):
        global logger
        logger.error(data)

def init_logging(debug_mode):
    if debug_mode:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.INFO)

    logfilehandler = logging.FileHandler(LOGFILE)

    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    logfilehandler.setFormatter(formatter)
    logger.addHandler(logfilehandler)

    syslogger = Syslogger()
    sys.stderr = syslogger

def logAgiFile(message, error = False):
    if error == False:
        logger.info(message)
    else:
        logger.error(message)

def send_request(url, customHeaders, params, auth):
    result = None
    #print "pre requete {} {} {}".format(url,params,customHeaders)
    try:
        result = requests.get(url,
                            headers = customHeaders,
                            params = params,
                            auth = auth,
                            timeout=CONNECTION_TIMEOUT)
        #agi.verbose("Request sent to server")

    except requests.exceptions.ConnectionError:
        #print 'Cnx error'
        logAgiFile("ACS Server refused connection")

    except requests.exceptions.Timeout:
        #print 'Timeout'
        logAgiFile("Timeout when connecting to the ACS server.")

    except:
        #print 'Uknown error'
        logAgiFile("Unknown problem during connection to the ACS server.")
        exc_type, exc_value, exc_traceback = sys.exc_info()
        logger.debug(repr(traceback.format_exception(exc_type, exc_value,
                                                     exc_traceback)))
    finally:
        return result


def get_channels():
    ari_auth = (ARI_USER, ARI_PASS)
    headers = {'content-type': 'application/json'}
    logAgiFile("Channels list request to ARI")
    res = send_request(channelsURL, headers, None, ari_auth)
    #print res.json()
    return res.json()

def get_rights(src_number, dst_number, token):
    params = {'sourceSymbolicAddress': src_number, 'destinationSymbolicAddress': dst_number}
    headers = {'content-type': 'application/json', 'token': token}
    logAgiFile("Precedence request for src_number: " + str(src_number) + "to dst_number: " + str(dst_number))
    res = send_request(PrecedenceRightsURL, headers, params)
    logAgiFile(res.json())
    return res.json()

def main():
    #agi.verbose('---------------------------------------------------')
    init_logging(DEBUG_MODE)

    try:
        channels_list = get_channels()
        channels_precedences = {'Routine': [], 'Priority': [], 'Immediate': [], 'FlashOverride': [], 'Flash': []}
        for c in channels_list:
            #print c
            #print '-----------------------------------------------------\n'
            print('id: {}, precedence: {}'.format(c['id'],c['accountcode']))
            if c['accountcode'] == 'Routine':
                channels_precedences['Routine'].append({'id': c['id']})
            elif  c['accountcode'] == 'Priority':
                channels_precedences['Priority'].append({'id': c['id']})
            elif  c['accountcode'] == 'Immediate':
                channels_precedences['Immediate'].append({'id': c['id']})
            elif  c['accountcode'] == 'FlashOverride':
                channels_precedences['FlashOverride'].append({'id': c['id']})
            elif  c['accountcode'] == 'Flash':
                channels_precedences['Flash'].append({'id': c['id']})

            my_precedence = agi.get_variable('agi_call_precedence')

            if precedencesPrio[my_precedence] > 1:
                if channels_precedences['Routine']:
                    logger.info('Raccrocher un canal de prio 1')
                    agi.verbose('Raccrocher un appel P1')
            
    except Exception:
        exc_type, exc_value, exc_traceback = sys.exc_info()
        logger.error(repr(traceback.format_exception(exc_type, exc_value,
                                          exc_traceback)))
        sys.exit(1)

    sys.exit(1)


if __name__ == '__main__':
    main()
